@@ -1,12 +1,11 @@
 package HTML::Template::Compiled::Parser;
-# $Id: Parser.pm 1154 2012-04-22 17:21:53Z tinita $
 use Carp qw(croak carp confess);
 use strict;
 use warnings;
 use base qw(Exporter);
 use HTML::Template::Compiled::Token qw(:tagtypes);
 use Scalar::Util;
-our $VERSION = 0.14;
+our $VERSION = '1.002'; # VERSION
 my @vars;
 BEGIN {
 @vars = qw(
@@ -327,7 +326,7 @@ sub find_attributes {
         ]);
         push @{$arg->{tags}}, $token;
         if ($token->is_open &&
-            not exists
+            exists
                 $self->get_tagnames->{CLOSING_TAG()}->{ $arg->{name} }) {
             $arg->{level}++
         }
